package service

import (
	"fmt"

	"github.com/ankitrgadiya/operatorlib/pkg/meta"
	"github.com/imdario/mergo"
	"github.com/pkg/errors"
	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
)

func (c Conf) Mutate() controllerutil.MutateFn {

	return func() error {
		out := c.Obj
		in := c.Obj.DeepCopyObject()

		om, err := meta.MutateObjectMeta(c)

		ports, err := c.GenServicePortsFunc()
		if err != nil {
			return errors.Wrap(err, "failed to generate service ports")
		}

		selectors, err := c.GenSelectorFunc()
		if err != nil {
			return errors.Wrap(err, "failed to generate service selectors")
		}

		out.ObjectMeta = *om
		out.Spec = corev1.ServiceSpec{
			Ports:    ports,
			Selector: selectors,
			Type:     corev1.ServiceType(c.Type),
		}

		fmt.Println(in)
		mergo.Merge(out, in)

		return err
	}
}

func (c Conf) MakeObject() *corev1.Service {
	obj := &corev1.Service{
		ObjectMeta: metav1.ObjectMeta{
			Name:      c.ObjectName(),
			Namespace: c.ObjectNamespace(),
		},
	}

	return obj
}
